'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _sourceMapSupport2 = require('source-map-support');

exports.default = WebDriver;

var _events = require('events');

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _sourceMapSupport2.install)();


var SCOPE_TYPES = {
    'browser': function Browser() {},
    'element': function Element() {}
};

function WebDriver(options, modifier) {
    var propertiesObject = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    /**
     * In order to allow named scopes for elements we have to propagate that
     * info within the `propertiesObject` object. This doesn't have any functional
     * advantages just provides better description of objects when debugging them
     */
    var scopeType = SCOPE_TYPES[propertiesObject.scope] || SCOPE_TYPES['browser'];
    delete propertiesObject.scope;

    var prototype = (0, _create2.default)(scopeType.prototype, {
        isW3C: { value: options.isW3C }
    });
    var log = (0, _wdioLogger2.default)('webdriver');

    var eventHandler = new _events.EventEmitter();
    var EVENTHANDLER_FUNCTIONS = (0, _getPrototypeOf2.default)(eventHandler);

    /**
     * WebDriver monad
     */
    function unit(sessionId, commandWrapper) {
        propertiesObject.commandList = { value: (0, _keys2.default)(propertiesObject) };
        propertiesObject.options = { value: options

            /**
             * allow to wrap commands if necessary
             * e.g. in wdio-cli to make them synchronous
             */
        };if (typeof commandWrapper === 'function') {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)((0, _entries2.default)(propertiesObject)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var _ref = _step.value;

                    var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

                    var commandName = _ref2[0];
                    var value = _ref2[1].value;

                    if (typeof value !== 'function') {
                        continue;
                    }

                    propertiesObject[commandName].value = commandWrapper(commandName, value);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }

        var client = (0, _create2.default)(prototype, propertiesObject);
        client.sessionId = sessionId;

        /**
         * register capabilities only to browser scope
         */
        if (scopeType.name === 'Browser') {
            client.capabilities = options.capabilities;
        }

        if (typeof modifier === 'function') {
            client = modifier(client, options);
        }

        client.addCommand = function (name, func) {
            unit.lift(name, func);
        };

        return client;
    }

    unit.lift = function (name, func) {
        prototype[name] = function next() {
            var client = unit(this.sessionId);

            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            log.info('COMMAND', `${name}(${args.join(', ')})`);

            /**
             * set name of function for better error stack
             */
            Object.defineProperty(func, 'name', { writable: true });
            func.name = name;

            return func.apply(client, args);
        };
    };

    /**
     * register event emitter
     */

    var _loop = function _loop(eventCommand) {
        prototype[eventCommand] = function () {
            eventHandler[eventCommand].apply(eventHandler, arguments);
            return this;
        };
    };

    for (var eventCommand in EVENTHANDLER_FUNCTIONS) {
        _loop(eventCommand);
    }

    return unit;
}
module.exports = exports['default'];